/*
 * This file and its contents are supplied under the terms of the
 * Common Development and Distribution License ("CDDL"), version 1.0.
 * You may only use this file in accordance with the terms of version
 * 1.0 of the CDDL.
 *
 * A full copy of the text of the CDDL should have accompanied this
 * source.  A copy of the CDDL is also available via the Internet at
 * http://www.illumos.org/license/CDDL.
 */

/*
 * Copyright 2012 David Hoeppner.  All rights reserved.
 */

#ifndef	_INET_DCCP_DCCP_STACK_H
#define	_INET_DCCP_DCCP_STACK_H

#include <sys/netstack.h>
#include <sys/cpuvar.h>
#include <sys/sunddi.h>
#include <sys/sunldi.h>
#include <inet/ip.h>
#include <inet/ipdrop.h>
#include <inet/dccp_stats.h>

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * DCCP stack instances
 */
typedef struct dccp_stack {
	netstack_t		*dccps_netstack;	/* Common netstack */

	uint_t			dccps_bind_fanout_size;
	struct dccp_df_s	*dccps_bind_fanout;

	/*
	 * Privileged ports.
	 */
#define	DCCP_NUM_EPRIV_PORTS	64
	int			dccps_num_epriv_ports;
	in_port_t		dccps_epriv_ports[DCCP_NUM_EPRIV_PORTS];
	kmutex_t		dccps_epriv_port_lock;

	in_port_t		dccps_min_anonpriv_port;
	uint_t			dccps_next_port_to_try;

	/* Reset rate control */
	int64_t			dccps_last_rst_intrvl;
	uint32_t		dccps_rst_cnt;

	/* Tunables table */
	struct mod_prop_info_s	*dccps_propinfo_tbl;

	ldi_ident_t		dccps_ldi_ident;

	/* MIB-II kernel statistics */
	kstat_t			*dccps_mibkp;
	kstat_t			*dccps_kstat;

	/* CPU stats counter */
	dccp_stats_cpu_t	**dccps_sc;
	int			dccps_sc_cnt;
} dccp_stack_t;

#ifdef	__cplusplus
}
#endif

#endif	/* _INET_DCCP_DCCP_STACK_H */
